searchData={"content_type":"text/markdown","items":[{"doc":"Skema is a simple schema validation and casting library for Elixir.\nSkema provide 3 main APIs:\n\n1. `Skema.cast_and_validate/2` for casting and validating data with given schema\n2. `Skema.cast/2` for casting data with given schema\n3. `Skema.validate/2` for validating data with given schema","ref":"Skema.html","title":"Skema","type":"module"},{"doc":"Skema schema could be a map with field name as key and field definition as value or a schema module.\n\n```elixir\nschema = %{\n  email: [type: :string, required: true],\n  age: [type: :integer, number: [min: 18]],\n  hobbies: [type: {:array, :string}]\n}\n```\nor\n```elixir\ndefmodule UserSchema do\n  use Skema\n\n  defschema do\n    field :email, :string, required: true\n    field :age, :integer, number: [min: 18]\n    field :hobbies, {:array, :string}\n  end\nend\n```","ref":"Skema.html#module-define-schema","title":"Define schema - Skema","type":"module"},{"doc":"You can use schema to cast and validate data like this\n\n```elixir\ndata = %{email: \"blue\", age: 10, hobbies: [\"swimming\", \"reading\"]}\nschema = %{\n  email: [type: :string, required: true],\n  age: [type: :integer, number: [min: 18]],\n  hobbies: [type: {:array, :string}]\n}\n\ncase Skema.cast_and_validate(data, schema) do\n  {:ok, data} -> IO.puts(\"Data is valid\")\n  {:error, errors} -> IO.puts(inspect(errors))\nend\n```","ref":"Skema.html#module-use-schema","title":"Use schema - Skema","type":"module"},{"doc":"- `Skema.cast_and_validate/2` will return error if there is any error in casting or validating data\n  - `Skema.cast/2` will return error for casting data only (data can be casted to proper type or not)\n  - `Skema.validate/2` will return error for validating data only","ref":"Skema.html#module-what-is-the-difference-between-apis-error","title":"What is the difference between APIs error - Skema","type":"module"},{"doc":"Cast data with given schema.","ref":"Skema.html#cast/2","title":"Skema.cast/2","type":"function"},{"doc":"Cast and validate data with given schema.","ref":"Skema.html#cast_and_validate/2","title":"Skema.cast_and_validate/2","type":"function"},{"doc":"","ref":"Skema.html#cast_array/3","title":"Skema.cast_array/3","type":"function"},{"doc":"","ref":"Skema.html#transform/1","title":"Skema.transform/1","type":"function"},{"doc":"","ref":"Skema.html#transform/2","title":"Skema.transform/2","type":"function"},{"doc":"Validate data with given schema.","ref":"Skema.html#validate/2","title":"Skema.validate/2","type":"function"},{"doc":"Result Struct for Skema operations","ref":"Skema.Result.html","title":"Skema.Result","type":"module"},{"doc":"Get error from result for given field.","ref":"Skema.Result.html#get_error/2","title":"Skema.Result.get_error/2","type":"function"},{"doc":"Create a new Result struct with given schema map and params.","ref":"Skema.Result.html#new/1","title":"Skema.Result.new/1","type":"function"},{"doc":"Put valid data to result.","ref":"Skema.Result.html#put_data/3","title":"Skema.Result.put_data/3","type":"function"},{"doc":"Put error to result.","ref":"Skema.Result.html#put_error/3","title":"Skema.Result.put_error/3","type":"function"},{"doc":"Borrow from https://github.com/ejpcmac/typed_struct/blob/main/lib/typed_struct.ex\n\nSchema specifications:\nInternal presentation of a schema is just a map with field names as keys and field options as values.\n\n**Example**\n\n```elixir\n%{\n  name: [type: :string, format: ~r/{4}/],\n  age: [type: :integer, number: [min: 15, max: 50]].\n  skill: [type: {:array, :string}, length: [min: 1, max: 10]]\n}\n```","ref":"Skema.Schema.html","title":"Skema.Schema","type":"module"},{"doc":"**Built-in types**\n\nA type could be any of built-in supported types:\n\n- `boolean`\n- `string` | `binary`\n- `integer`\n- `float`\n- `number` (integer or float)\n- `date`\n- `time`\n- `datetime` | `utc_datetime`: date time with time zone\n- `naive_datetime`: date time without time zone\n- `map`\n- `keyword`\n- `{array, type}` array of built-in type, all item must be the same type\n\n\n**Other types**\nCustom type may be supported depends on module.\n\n**Nested types**\nNested types could be a another **schema** or list of **schema**\n\n\n```elixir\n%{\n  user: [type: %{\n      name: [type: :string]\n    }]\n}\n```\n\nOr list of schema\n\n```elixir\n%{\n  users: [type: {:array, %{\n      name: [type: :string]\n    }} ]\n}\n```","ref":"Skema.Schema.html#module-i-field-type","title":"I. Field type - Skema.Schema","type":"module"},{"doc":"These specifications is used for casting data with `Skema.Params.cast`\n\n#","ref":"Skema.Schema.html#module-ii-field-casting-and-default-value","title":"II. Field casting and default value - Skema.Schema","type":"module"},{"doc":"Is used when the given field is missing or nil.\n\n- Default could be a value\n\n  ```elixir\n  %{\n    status: [type: :string, default: \"active\"]\n  }\n  ```\n\n- Or a `function/0`, this function will be invoke each time data is `casted`\n\n  ```elixir\n  %{\n    published_at: [type: :datetime, default: &DateTime.utc_now/0]\n  }\n  ```\n\n#","ref":"Skema.Schema.html#module-1-default-value","title":"1. Default value - Skema.Schema","type":"module"},{"doc":"You can provide a function to cast field value instead of using default casting function by using\n`cast_func:  `\n\n```elixir\n%{\n    published_at: [type: :datetime, cast_func: &DateTime.from_iso8601/1]\n}\n```","ref":"Skema.Schema.html#module-2-custom-cast-function","title":"2. Custom cast function - Skema.Schema","type":"module"},{"doc":"**These validation are supported by [valdi](https://hex.pm/packages/valdi)**\n\n**Custom validation function**\n\nYou can provide a function to validate the value.\n\nDefine validation: `func:  `\n\nFunction must be follow this signature\n\n```elixir\n@spec func(value::any()) :: :ok | {:error, message::String.t()}\n```","ref":"Skema.Schema.html#module-iii-field-validation","title":"III. Field validation - Skema.Schema","type":"module"},{"doc":"`defschema` helps you define schema clearly and easy to use.\n\n```elixir\ndefmodule MyStruct do\n  use Skema.Schema\n\n  defschema do\n    field :field_one, :string\n    field :field_two, :integer, required: true\n    field :field_four, :atom, default: :hey\n    field :update_time, :naive_datetime, default: &NaiveDateTime.utc_now/0\n  end\nend\n```","ref":"Skema.Schema.html#module-define-schema-with-defschema-macro","title":"Define schema with `defschema` macro - Skema.Schema","type":"module"},{"doc":"Defines a typed struct.\n\nInside a `defschema` block, each field is defined through the `field/2`\nmacro.","ref":"Skema.Schema.html#defschema/2","title":"Skema.Schema.defschema/2","type":"macro"},{"doc":"defmodule MyStruct do\n      use Skema.Schema\n\n      defschema do\n        field :field_one, :string\n        field :field_three, :boolean, required: true\n        field :field_four, :atom, default: :hey\n        field :update_time, :naive_datetime, default: &NaiveDateTime.utc_now/0\n      end\n    end\n\nYou can create the struct in a submodule instead:\n\n    defmodule MyModule do\n      use Skema.Schema\n\n      defschema Comment do\n        field :user_id, :integer, required: true\n        field :content, :string, required: true\n      end\n\n      defschema Post do\n        field :field_one, :string\n        field :field_two, :integer, required: true\n        field :field_three, :boolean, required: true\n        field :field_four, :string, default: \"hello\"\n        field :update_time, :naive_datetime, default: &NaiveDateTime.utc_now/0\n        field :comment, Comment, required: true\n      end\n    end\n\n    MyModule.Post.cast(%{field_two: 1, field_three: true, comment: %{user_id: 1, content: \"hello\"}})","ref":"Skema.Schema.html#defschema/2-examples","title":"Examples - Skema.Schema.defschema/2","type":"macro"},{"doc":"Defines a field in a typed struct.","ref":"Skema.Schema.html#field/3","title":"Skema.Schema.field/3","type":"macro"},{"doc":"# A field named :example of type String.t()\n    field :example, String.t()","ref":"Skema.Schema.html#field/3-example","title":"Example - Skema.Schema.field/3","type":"macro"},{"doc":"* `default` - sets the default value for the field\n  * `required` - if set to true, enforces the field and makes its type\n    non-nullable","ref":"Skema.Schema.html#field/3-options","title":"Options - Skema.Schema.field/3","type":"macro"},{"doc":"Cast data to target type.\nMuch code of this module is borrowed from `Ecto.Type`","ref":"Skema.Type.html","title":"Skema.Type","type":"module"},{"doc":"","ref":"Skema.Type.html#cast/2","title":"Skema.Type.cast/2","type":"function"},{"doc":"","ref":"Skema.Type.html#cast!/2","title":"Skema.Type.cast!/2","type":"function"},{"doc":"# Skema\n\nPhoenix request params validation library.\n\n[![Build Status](https://github.com/bluzky/skema/workflows/Elixir%20CI/badge.svg)](https://github.com/bluzky/skema/actions) [![Coverage Status](https://coveralls.io/repos/github/bluzky/skema/badge.svg?branch=main)](https://coveralls.io/github/bluzky/skema?branch=main) [![Hex Version](https://img.shields.io/hexpm/v/skema.svg)](https://hex.pm/packages/skema) [![docs](https://img.shields.io/badge/docs-hexpm-blue.svg)](https://hexdocs.pm/skema/)\n\n\n\n- [Skema](#skema)\n    - [Why Skema](#why-skema)\n    - [Installation](#installation)\n    - [Usage](#usage)\n    - [Define schema](#define-schema)\n        - [Default value](#default-value)\n        - [Custom cast function](#custom-cast-function)\n            - [1. Custom cast function accept value only](#1-custom-cast-fuction-accept-value-only)\n            - [2.Custom cast function accept tuple {M, f}](#3custom-cast-function-accept-tuple-m-f)\n        - [Nested schema](#nested-schema)\n    - [Validation](#validation)\n    - [Contributors](#contributors)","ref":"readme.html","title":"Skema","type":"extras"},{"doc":"- Reduce code boilerplate \n    - Shorter schema definition\n    - Default function which generate value each casting time\n    - Custom validation functions\n    - Custom parse functions","ref":"readme.html#why-skema","title":"Why Skema - Skema","type":"extras"},{"doc":"[Available in Hex](https://hex.pm/skema), the package can be installed\nby adding `skema` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:skema, \"~> 0.1\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - Skema","type":"extras"},{"doc":"**Process order**\n> Cast data -> validate casted data -> transform data\n\n```elixir\nuse Skema\ndefschema IndexParams do\n    field :keyword, :string\n    field :status, :string, required: true\n    field :group_id, :integer, number: [greater_than: 0]\n    field :name, :string\nend\n\ndef index(conn, params) do\n    with {:ok, better_params} <- Skema.cast(params, IndexParams) do\n        # do anything with your params\n    else\n        {:error, errors} -> # return params error\n    end\nend\n```","ref":"readme.html#usage","title":"Usage - Skema","type":"extras"},{"doc":"```elixir\nuse Skema\n\ndefschema IndexParams do\n    field :keyword, :string\n    field :status, :string, required: true\n    field :group_id, :integer, number: [greater_than: 0]\n    field :name, :string\nend\n```\n\nDefine a field using macro `@spec field(field_name :: atom(), type :: term(), opts \\\\ [])`\n\n- `type`: `Skema` support same data type as `Ecto`. I borrowed code from Ecto\n\nSupported options:\n\n- `default`: default value or default function\n- `cast_func`: custom cast function\n- `number, format, length, in, not_in, func, required, each` are available validations\n- `from`: use value from another field\n- `as`: alias key you will receive from `Skema.cast` if casting is succeeded\n\n\n#","ref":"readme.html#define-schema","title":"Define schema - Skema","type":"extras"},{"doc":"You can define a default value for a field if it's missing from the params.\n\n```elixir\nfield :status, :string, default: \"pending\"\n```\n\nOr you can define a default value as a function. This function is evaluated when `Skema.cast` gets invoked.\n\n```elixir\nfield :date, :utc_datetime, default: &Timex.now/0\n```\n\n#","ref":"readme.html#default-value","title":"Default value - Skema","type":"extras"},{"doc":"You can define your own casting function, `skema` provide `cast_func` option.\nYour `cast_func` must follows this spec \n\n##","ref":"readme.html#custom-cast-function","title":"Custom cast function - Skema","type":"extras"},{"doc":"```elixir\nfn(any) :: {:ok, any} | {:error, binary} | :error\n```\n\n```elixir\ndef my_array_parser(value) do\n    if is_binary(value) do\n        ids = \n            String.split(value, \",\")\n            |> Enum.map(&String.to_integer(&1))\n        \n        {:ok, ids}\n    else\n        {:error, \"Invalid string\"\n    end\nend\n\ndefschema Sample do\n   field :user_id, {:array, :integer}, cast_func: &my_array_parser/1\nend\n\n```\nThis is a demo parser function.\n\n\n#","ref":"readme.html#1-custom-cast-fuction-accept-value-only","title":"1. Custom cast fuction accept value only - Skema","type":"extras"},{"doc":"With `Skema` you can parse and validate nested map and list easily\n\n```elixir\ndefschema Address do\n    field :street, :string\n    field :district, :string\n    field :city, :string\nend\n\ndefschema User do\n    field :name, :string,\n    field :email, :string, required: true\n    field :addresses, {:array, Address}\nend\n```","ref":"readme.html#nested-schema","title":"Nested schema - Skema","type":"extras"},{"doc":"`Skema` uses `Valdi` validation library. You can read more about [Valdi here](https://github.com/bluzky/valdi)\nBasically it supports following validation\n\n- validate inclusion/exclusion\n- validate length for string and enumerable types\n- validate number\n- validate string format/pattern\n- validate custom function\n- validate required(not nil) or not\n- validate each array item\n\n\n\n  ```elixir\ndefschema Product do\n    field :sku, :string, required: true, length: [min: 6, max: 20]\n    field :name, :string, required: true,\n    field :quantity, :integer, number: [min: 0],\n    field :type: :string, in: ~w(physical digital),\n    field :expiration_date, :naive_datetime, func: &my_validation_func/1,\n    field :tags, {:array, :string}, each: [length: [max: 50]]\nend\n  ```\n\n#","ref":"readme.html#validation","title":"Validation - Skema","type":"extras"},{"doc":"- Can accept function or `{module, function}` tuple\n- Only support 2 arity function\n\n\n```elixir\ndef require_email?(value, data), do: is_nil(email.phone)\n\n....\n\nfield :email, :string, required: {__MODULE__, :require_email?}\n```\n\n#","ref":"readme.html#dynamic-required","title":"Dynamic required - Skema","type":"extras"},{"doc":"Support validate array item with `:each` option, `each` accept a list of validators\n\n```elixir\n...\n    field :values, {:array, :number}, each: [number: [min: 20, max: 50]]\n...\n```","ref":"readme.html#validate-array-item","title":"Validate array item - Skema","type":"extras"},{"doc":"If you find a bug or want to improve something, please send a pull request. Thank you!","ref":"readme.html#thank-you","title":"Thank you - Skema","type":"extras"}]}